/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/
#include "global.h"
#include "ArgParser.h"

#include <sstream>
#include <iostream>

void ArgParser::AddArg(struct Arg *a)
{
  assert(a);
  assert(Args.find(a->name) == Args.end());
  Args[a->name] = a;
}

void ArgParser::Parse(istream &str, unsigned lineNum)
{
  string argName;

  while (1) {
    str >> argName;
    if (argName.empty()) {
      return;
    }
    ArgMap::iterator ami = Args.find(argName);
    if (ami != Args.end()) {
      ami->second->Parse(str);
    } else {
      cerr << "Parse error on line " << lineNum << "!!!" << endl;
      cerr << "Argument parser found token \"" << argName 
           << "\" but that's not a recognized option." << endl;
      cerr << "Recognized options are:";
      for (ami = Args.begin(); ami != Args.end(); ami++) {
        cerr << " " << ami->second->name;
      }
      cerr << endl;
      exit(1);
    }
    argName.clear();
  }
}
