/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/

/* Port Protocol Migration state machine : 17.26 */
 
#include "global.h"
#include "STPM.h"
#include "ProtocolMigration.h"
#include "Port.h"

#define STATES { \
  CHOOSE(INIT),     \
  CHOOSE(SEND_RSTP),    \
  CHOOSE(SENDING_RSTP), \
  CHOOSE(SEND_STP), \
  CHOOSE(SENDING_STP),  \
}

#define GET_STATE_NAME ProtocolMigration::GetStateName
#include "choose.h"

#define MigrateTime 3 /* 17,16.4 */

ProtocolMigration::ProtocolMigration(Port *p) : 
  StateMachine("ProtocolMigration"), owner(p)
{
  // empty
}

void ProtocolMigration::EnterState()
{
  Port *port = owner;

  switch (State) {
    case BEGIN:
    case INIT:
      port->initPm = true;
      port->mcheck = false;
      break;
    case SEND_RSTP:
      port->mdelayWhile = MigrateTime;
      port->mcheck = port->initPm = false;
      port->sendRSTP = true;
      break;
    case SENDING_RSTP:
      port->rcvdRSTP = port->rcvdSTP = false;
      break;
    case SEND_STP:
      port->mdelayWhile = MigrateTime;
      port->sendRSTP = false;
      port->initPm = false;
      break;
    case SENDING_STP:
      port->rcvdRSTP = port->rcvdSTP = false;
      break;
  }
}

bool ProtocolMigration::CheckCondition()
{
  Port *port = owner;

  if ((!port->portEnabled && !port->initPm) || BEGIN == State)
    return Hop2State(INIT);

  switch (State) {
    case INIT:
      if (port->portEnabled) {
        return Hop2State ((port->owner->GetForceVersion() >= 2) ?
                          SEND_RSTP : SEND_STP);
      }
      break;
    case SEND_RSTP:
      return Hop2State(SENDING_RSTP);
    case SENDING_RSTP:
      if (port->mcheck)
        return Hop2State(SEND_RSTP);
      if (port->mdelayWhile &&
          (port->rcvdSTP || port->rcvdRSTP)) {
        return Hop2State(SENDING_RSTP);
      }
        
      if (!port->mdelayWhile && port->rcvdSTP) {
        return Hop2State(SEND_STP);       
      }
        
      if (port->owner->GetForceVersion() < 2) {
        return Hop2State(SEND_STP);
      }
        
      break;
    case SEND_STP:
      return Hop2State(SENDING_STP);
    case SENDING_STP:
      if (port->mcheck)
        return Hop2State(SEND_RSTP);
      if (port->mdelayWhile &&
          (port->rcvdSTP || port->rcvdRSTP))
        return Hop2State(SENDING_STP);
      if (!port->mdelayWhile && port->rcvdRSTP)
        return Hop2State(SEND_RSTP);
      break;
  }
  return false;
}

