/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/
#include "global.h"
#include "Switch.h"
#include "Log.h"
#include "Parser.h"
#include "Simulator.h"

#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#include <iostream>

void printUsage(char *prog)
{
  cerr << prog << ": usage: " << endl
       << prog << " -f simulationFile [-h] [-l logfile]" << endl;
}

int main(int argc, char *argv[])
{
  extern char *optarg;
  int rc;
  char *filename = NULL;
  char *logfile = NULL;

  while ((rc = getopt(argc, argv, "f:hl:")) != -1) {
    switch (rc) {
    case 'f':
      filename = strdup(optarg);
      assert(filename);
      break;
    case 'l':
      logfile = strdup(optarg);
      assert(logfile);
      break;
    case 'h':
    default:
      printUsage(argv[0]);
      exit(1);
    }
  }

  if (filename == NULL) {
    cerr << "Error! You must provide a simulation file on the command line." 
         << endl;
    printUsage(argv[0]);
    exit(1);
  }

  log_init(logfile);

  stp_trace("simulation scenario file is %s", filename);

  Simulator simulator;
  Parser parser(&simulator);
  parser.Parse(filename);
  
  simulator.Run();

  log_close();

  free(filename);
  filename = NULL;

  if (logfile) {
    free(logfile);
    logfile = NULL;
  }
}

