/************************************************************************ 
 * RSTP library - Rapid Spanning Tree (802.1t, 802.1w) 
 * Copyright (C) 2001-2003 Optical Access 
 * Author: Alex Rozin 
 * 
 * Portions copyright (C) 2004 Andy Myers, who modified this code to
 * be C++ and wrapped it in BridgeSim
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; version 2.1
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with RSTP library; see the file COPYING.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 **********************************************************************/

/* This file contains system dependent API
   from the RStp to a operation system (see stp_to.h) */

/* stp_to API for Linux */

#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>

#include "global.h"
#include "STPM.h"
#include "stp_to.h"
#include "RapidSpanningTree.h"
#include "Log.h"

// #ifdef STRONGLY_SPEC_802_1W
int
STP_OUT_set_learning (int port_index, int vlan_id, int enable)
{
  const char *str = enable ? "enabled" : "disabled";
  stp_trace("set_learning port %d %s", port_index, str);
  return STP_OK;
}

int
STP_OUT_set_forwarding (int port_index, int vlan_id, int enable)
{
  const char *str = enable ? "enabled" : "disabled";
  stp_trace("set_forwarding port %d %s", port_index, str);
  return STP_OK;
}
// #else // STRONGLY_SPEC_802_1W
/* 
 * In many kinds of hardware the state of ports may
 * be changed with another method
 */
int
STP_OUT_set_port_state (IN int port_index, IN int vlan_id,
            IN RSTP_PORT_STATE state)
{
  return STP_OK;
  //return AR_INT_STP_set_port_state (port_index, vlan_id, state);
}
// #endif // STRONGLY_SPEC_802_1W


int
STP_OUT_flush_lt (IN int port_index, IN int vlan_id, LT_FLASH_TYPE_T type, char* reason)
{
  const char *exclude = 
    (type == LT_FLASH_ALL_PORTS_EXCLUDE_THIS) ? "Exclude" : "Only";
  stp_trace("clearFDB port %d (%s) reason %s", port_index, exclude, reason);

  return STP_OK;
}

int
STP_OUT_set_hardware_mode (int vlan_id, UID_STP_MODE_T mode)
{
  return STP_OK;
  //return AR_INT_STP_set_mode (vlan_id, mode);
}


const char *
STP_OUT_get_port_name (IN int port_index)
{
  static char tmp[4];
  sprintf (tmp, "p%02d", (int) port_index);
  return tmp;
  //return port2str (port_index, &sys_config);
}

unsigned long
STP_OUT_get_deafult_port_path_cost (IN unsigned int portNo)
{
  return 20000;
}

unsigned long STP_OUT_get_port_oper_speed (unsigned int portNo)
{
  /*
    if (portNo <= 2)
    return 1000000L;
    else
    return 1000L;
  */
  return 1000000L;
}

int             /* 1- Full, 0- Half */
STP_OUT_get_duplex (IN int port_index)
{
  return 1;
}

int
STP_OUT_get_init_stpm_cfg (IN int vlan_id,
                           INOUT UID_STP_CFG_T* cfg)
{
  cfg->bridge_priority =        DEF_BR_PRIO;
  cfg->max_age =                DEF_BR_MAXAGE;
  cfg->hello_time =             DEF_BR_HELLOT;
  cfg->forward_delay =          DEF_BR_FWDELAY;
  cfg->force_version =          NORMAL_RSTP;

  return STP_OK;
}
  

int
STP_OUT_get_init_port_cfg (IN int vlan_id,
                           IN int port_index,
                           INOUT UID_STP_PORT_CFG_T* cfg)
{
  cfg->port_priority =                  DEF_PORT_PRIO;
  cfg->admin_non_stp =                  DEF_ADMIN_NON_STP;
  cfg->admin_edge =                     DEF_ADMIN_EDGE;
  cfg->admin_port_path_cost =           ADMIN_PORT_PATH_COST_AUTO;
  cfg->admin_point2point =              DEF_P2P;

  return STP_OK;
}



